#include <UTF8.h>

#ifndef UTFCHAR
#define UTFCHAR

//Current max UTF Char byte length (with a null byte added)
#define MAX_UTF_CHARLEN 5

//Little function used to calculate the byte length of a UTF Character
inline uint8 UTF8CHARBYTELEN(const char tchar)
{
	uint8 len;
	
	len = (((0xE5000000 >> ((tchar >> 3) & 0x1E)) & 3) + 1);
		
	return len;
}

#pragma export on

/*
	CLASS UTFChar
	
	A class that encapsulates the concept of a UTF encoded character.
	
	A UTF Character can encompass from one to four bytes. Therefore, 
	it is easier to just wrap this variable byte length with a class.
*/

class UTFChar : public BLocker
{
public:
	//Constructors
	UTFChar();
	UTFChar(const char*);
	UTFChar(const UTFChar&);
	
	//Destructors
	virtual ~UTFChar();
	
	//Copy operators
	UTFChar& operator=(const char*);
	UTFChar& operator=(const UTFChar&);
	
	//Casting operator
	operator const char*() const;
	
	//Equivelency Operators
	bool operator==(const char*) const;
	bool operator==(const UTFChar&) const;
	
	bool operator!=(const char*) const;
	bool operator!=(const UTFChar&) const;
	
	//Methods
	
	/*
		ByteLength
		
		Returns the byte length of the current character
	*/
	uint8 ByteLength();
	
	/*
		IsAlphaNum
		
		Returns a bool stating whther this character is an alpha
		numeric character. Alpha Numeric characters for this case are
		any ascii character that returns true from isalnum, or any multibyte
		character.
	*/
	bool IsAlphaNum();
	
	/*
		IsAscii
		
		Returns a bool stating that this UTFChar is an ascii character
		or not.
	*/
	bool IsAscii();
	
	/*
		IsNull
		
		Returns a bool stating whether or not this character is null
	*/
	bool IsNull();
		
protected:

private:

	void ClearBuf();
	
	//Buffer
	char m_buf[MAX_UTF_CHARLEN];

};

#pragma export off

#endif
