
class UTFChar;

#ifndef UTFSTRING
#define UTFSTRING

#pragma export on

/*
	CLASS UTFString
	
	A class to wrap the concept of a UTFString.
*/

class UTFString : BLocker
{
public:
	//Constructors
	UTFString();
	UTFString(const char*);
	UTFString(const UTFChar&);
	UTFString(const UTFString&);
	
	//Destructors
	virtual ~UTFString();
	
	//Copy Operators 
	UTFString& operator=(const char*);
	UTFString& operator=(const UTFString&);
	
	//Casting operator
	operator const char*() const;

	//Equivelency Operators
	bool operator==(const char*) const;
	bool operator==(const UTFString&) const;

	bool operator!=(const char*);
	bool operator!=(const UTFString&);
			
	//Concatenation operators
	UTFString& operator+=(const char*);
	UTFString& operator+=(const UTFString&);
	
	friend UTFString operator+(const UTFString&, const UTFString&);
	
	//Methods
	/*
		Length
		
		returns the character length of the UTF String.
	*/
	uint32 Length() const;
	
	/*
		ByteLength
		
		returns the bytelength of the UTF String.
	*/
	uint32 ByteLength() const;
	
	/*
		Insert
		
		Inserts a UTFString into this string given a starting and 
		ending positions within the string. Any characters within the
		start and end positions are replaced.
		
	*/
	void Insert(uint32 bindex, uint32 eindex, const UTFString&);

	/*
		Remove
		
		Removes a substring from this string, given the starting and
		ending positions.
		
		Returns the removed characters as a UTFString.
	*/
	UTFString Remove(uint32 bindex, uint32 eindex);
	
	/*
		Left
		
		Returns the Left n characters of the UTFString as a new UTFString.
		
		0 < n <= Length of the string.
	*/
	UTFString Left(uint32 n);

	/*
		Right
		
		Returns the Right n characters of the UTFString as a new UTFString.
		
		0 < n <= Length of the string.
	*/
	UTFString Right(uint32 n);
	
	/*
		Mid
		
		Returns a substring of this string given the starting position and 
		the total number of characters to be copied.
	
		0 <= n < Lemgth of string
		0 < c < Length of string
		
		if (n + c) > length of string, then c is capped so that
		(n + c) = length of string.
	*/
	UTFString Mid(uint32 n, uint32 c);
	
	/*
		Split
		
		Splits the string into two strings based on a character position
		or a given delimiter string.
		
		Split by position
		0 <= index < length of string
		
		Split by delimiter
		0 < length of delimiter  
	*/
	void Split(uint32 index, UTFString& lhs, UTFString& rhs);
	bool Split(const UTFString& delim, UTFString& lhs, UTFString& rhs);
	
	/*
		InStr
		
		Return the character position of a substring within this
		string. If a starting character position is given, then substring
		search begins at that character position in the string.
		
		0 < length of substring
		0 <= subpos < length of string
	*/
	int32 InStr(const UTFString& substr);
	int32 InStr(int subpos, const UTFString& substr);
	
	/*
		Replace
		
		Replaces all occurances of substring 1 inthis string
		with substring 2;
		
		0 < length of substring 1
		0 < length of substring 2
	*/
	void Replace(const UTFString& str1, const UTFString& str2);
	
	/*
		UTF replacement methods for common ASCII functions.
	
		NOTE: These only work on ASCII bytes in the UTF8 string!
		
	*/
	void ToUpper();
	void ToLower();
			
	/*
		GetChar
		
		Given a character position, returns the UTFChar at that position.
		
		0 <= index < length of string 
	*/
	UTFChar GetChar(uint32 index);
	
	/*
		SetChar
		
		Given a UTFChar and position, replace the UTF character at
		that position with a new one.
		
		0 <= index < length of string
		0 < length of utfchar
	*/
	void SetChar(const UTFChar& utfchar, uint32 index);

protected:
	
	
private:

	void CalcLength();
	void ClearBuffer();
	void CopyString(const char*);
	void StrCat(const char*);
		
	//Buffer
	char*	m_pBuffer;
	
	//Length
	uint32	m_length;
	
};

#pragma export off

#endif