#include "UTFChar.h"
#include "UTFString.h"
#include <List.h>
#include "UTFStringTokenizer.h"

UTFStringTokenizer::UTFStringTokenizer()
:	m_strDelim(" ")
{
}

UTFStringTokenizer::~UTFStringTokenizer()
{
	ClearTokens();
		
}
		
void UTFStringTokenizer::SetTokenString(const UTFString& tokstr)
{
	m_strToken = tokstr;
}

void UTFStringTokenizer::SetIgnoreString(const UTFString& igstr)
{
	m_strIgnore = igstr;
}

void UTFStringTokenizer::SetDelimiterString(const UTFString& delimstr)
{
	m_strDelim = delimstr;
}

const BList& UTFStringTokenizer::Tokenize()
{

	BAutolock(this);

	ClearTokens();
	
	if (0 >= m_strToken.Length())
		return m_tokens;
		
	UTFString tokens(m_strToken);
	UTFString lhs;
	UTFString rhs;
	
	UTFString *tok = NULL;
	UTFString *delim = NULL;
		
	bool delimok;
	
	delimok = true;
	
	if (-1 < m_strIgnore.InStr(m_strDelim))
		delimok = false;
		
	bool splok;
	
	splok = tokens.Split(m_strDelim, lhs, rhs);

	while (splok)
	{
		
		if (0 < lhs.Length())
		{
			if (-1 == m_strIgnore.InStr(lhs))
			{
				tok = new UTFString(lhs);
				m_tokens.AddItem((void*)tok);
			}
		}

		if (delimok)
		{
			delim = new UTFString(m_strDelim);
			m_tokens.AddItem((void*)delim);
		}
		
		tokens = rhs;		
		splok = tokens.Split(m_strDelim, lhs, rhs);
	}
	
	if (0 < lhs.Length())
	{
		if (-1 == m_strIgnore.InStr(lhs))
		{
			tok = new UTFString(lhs);
			m_tokens.AddItem((void*)tok);
		}
	}

	
	
	return m_tokens;
}

void UTFStringTokenizer::ClearTokens()
{
	UTFString *t;
	
	for (long i = 0; t = (UTFString*)m_tokens.ItemAt(i); i++)
		delete t;
		
	m_tokens.MakeEmpty();
}
