#include <assert.h>
#include "UTFChar.h"
#include "UTFString.h"
#include "UTFStringTokenizer.h"
#include "UTFTextBuffer.h"

//Constructors
UTFTextBuffer::UTFTextBuffer()
{
}

UTFTextBuffer::UTFTextBuffer(BDataIO* inbuf)
{
	ProcessDataBuffer(inbuf);
}
	
UTFTextBuffer::UTFTextBuffer(const UTFTextBuffer& inbuf)
{
	CopyTextBuffer(inbuf);
}

//Destructor
UTFTextBuffer::~UTFTextBuffer()
{
	ClearBuffer();
}

//Copy Operator
UTFTextBuffer& UTFTextBuffer::operator=(const UTFTextBuffer& ibuf)
{
	CopyTextBuffer(ibuf);
}

//Methods
void UTFTextBuffer::LoadData(BDataIO* loadbuf)
{
	ProcessDataBuffer(loadbuf);
}
	
void UTFTextBuffer::SaveData(BDataIO* loadbuf)
{
	for (long i = 0; UTFString* pt = (UTFString*)m_buffer.ItemAt(i); i++)
	{
		loadbuf->Write((const char*)*pt, pt->ByteLength());
	}
}

void UTFTextBuffer::SetCaret(uint32 Row, uint32 Col)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	assert(0 <= Col);
	
	BAutolock(this);

	m_carposy = Row;
	m_carposx = Col;
	
	ResetSelection();
	
}

void UTFTextBuffer::GetCaret(uint32& Row, uint32& Col)
{
	BAutolock(this);

	Row = m_carposy;
	Col = m_carposx;
	
}

void UTFTextBuffer::CaretUp()
{
	BAutolock(this);

	m_carposy--;
	
	if (m_carposy < 0)
		m_carposy = 0;
	
}

void UTFTextBuffer::CaretDown()
{
	BAutolock(this);

	m_carposy++;
	
	if (m_carposy >= m_buffer.CountItems())
		m_carposy = m_buffer.CountItems() - 1;
	
}

void UTFTextBuffer::CaretLeft()
{
	BAutolock(this);

	m_carposx--;
	if (m_carposx < 0)
	{
		if (0 == m_carposy)
		{
			m_carposx = 0;
		}
		else
		{
			m_carposy--;
			UTFString* ptstr = (UTFString*)m_buffer.ItemAt(m_carposy);
			m_carposx = ptstr->Length()-1;
		}
	}
	
}

void UTFTextBuffer::CaretRight()
{
	BAutolock(this);

	UTFString* ptstr = (UTFString*)m_buffer.ItemAt(m_carposy);

	m_carposx++;
	if (m_carposx >= ptstr->Length())
	{
		if (m_carposy == m_buffer.CountItems() - 1)
		{
			m_carposx = ptstr->Length() - 1;
		}
		else
		{
			m_carposy++;
			m_carposx = 0;
		}
	}
	
}

void UTFTextBuffer::SetSelectionBegin(uint32 Row, uint32 Col)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	assert(0 <= Col);

	BAutolock(this);

	
/*
	if (Row < m_selstarty)
	{
		m_selstarty = Row;
		m_selstartx = Col;
	}
	else if (Row == m_selstarty)
	{
		if (Col < m_selstartx)
		{
			m_selstartx = Col;
		}
		else
		{
			m_selendx = Col;
		}
	}
	else
	{
*/
		m_selstarty = Row;
		m_selstartx = Col;
//	}

	
}

void UTFTextBuffer::SetSelectionEnd(uint32 Row, uint32 Col)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	assert(0 <= Col);

	BAutolock(this);


/*
	if (Row < m_selstarty)
	{
		m_selstarty = Row;
		m_selstartx = Col;
	}
	else if (Row == m_selstarty)
	{
		if (Col < m_selstartx)
		{
			m_selstartx = Col;
		}
		else
		{
			m_selendx = Col;
		}
	}
	else
	{
*/
		m_selendy = Row;
		m_selendx = Col;
//	}

	
}

void UTFTextBuffer::ResetSelection()
{
	BAutolock(this);

	m_selstartx = m_selendx = m_carposx;
	m_selstarty = m_selendy = m_carposy;
	
}
	
void UTFTextBuffer::InsertChar(const UTFChar& InsChar)
{
	UTFString* tstr = NULL;
	UTFString lhs;
	UTFString rhs;
	
	BAutolock(this);

	Cut();
	
	if (0 >= m_buffer.CountItems())
	{
		UTFString* ptstr = new UTFString(InsChar);
		m_buffer.AddItem((void*)ptstr);
		return;
	}
	
	tstr = (UTFString*)m_buffer.ItemAt(m_carposy);
	
	//printf("(%3d, %3d): %s\n", m_carposy, m_carposx, (const char*)*tstr);
	
	tstr->Split(m_carposx, lhs, rhs);	

	if (InsChar == "\n")
	{
		*tstr = lhs;
		UTFString* ptstr = new UTFString(rhs);
		
		//printf("LHS (%3d, %3d): %s\n", m_carposy, m_carposx, (const char*)*tstr);
		m_carposy += 1;
		//printf("RHS (%3d, %3d): %s\n", m_carposy, m_carposx, (const char*)*ptstr);
		
		m_buffer.AddItem((void*)ptstr, m_carposy);
		m_carposx = 0;
	}
	else
	{
		*tstr = lhs + InsChar + rhs;
	
		m_carposx += 1;
	}
	
	ResetSelection();
	
}

UTFChar UTFTextBuffer::RemoveChar()
{
	BAutolock(this);

	if (0 >= m_buffer.CountItems())
	{
		UTFChar t;
		return t;
	}

	UTFString* ptstr = (UTFString*)m_buffer.ItemAt(m_carposy);
	
	UTFString lhs;
	UTFString rhs;
	
	ptstr->Remove(m_carposx, m_carposx+1);

	printf("%s\n", (const char*)*ptstr);
	
	if (0 >= ptstr->Length())
	{
		ptstr = (UTFString*)m_buffer.RemoveItem(m_carposy);
		delete ptstr;
	}
	
	
}

UTFString UTFTextBuffer::Cut()
{
	UTFString tstr;
	
	BAutolock(this);

	if (m_selstarty <= m_selendy)
	{
		UTFString* ptstr = NULL;
		UTFString lhs;
		UTFString rhs;
		UTFString tstr2;
		

		if (m_selstarty == m_selendy)
		{
			if (m_selstartx < m_selendx)
			{
			
				ptstr = (UTFString*)m_buffer.RemoveItem(m_selstarty);
				ptstr->Split(m_selendx, lhs, rhs);
				lhs.Split(m_selstartx, tstr, rhs);
				tstr2 = tstr + rhs;
				tstr = rhs;
				
				delete ptstr;

				if (0 < tstr2.Length())
				{
					ptstr = new UTFString(tstr2);
					m_buffer.AddItem((void*)ptstr, m_selstarty);	
				}
			}	
		}
		else
		{
			
			ptstr = (UTFString*)m_buffer.RemoveItem(m_selstarty);
			ptstr->Split(m_selstartx, lhs, rhs);
			tstr2 = lhs;
			tstr = rhs + "\n";

			delete ptstr;
		
			for(long i = m_selstarty + 1; i < m_selendy; i++)
			{
				ptstr = (UTFString*)m_buffer.RemoveItem(m_selstarty);
				tstr += *ptstr;
				tstr += "\n";
				delete ptstr;
			}

			ptstr = (UTFString*)m_buffer.RemoveItem(m_selstarty);
			ptstr->Split(m_selendx, lhs, rhs);
			tstr += lhs;
			tstr += "\n";
			tstr2 += rhs;
			
			delete ptstr;
		
			ptstr = new UTFString(tstr2);
			
			m_buffer.AddItem((void*)ptstr, m_selstarty);	
		}
	}
	
	m_carposx = m_selstartx;
	m_carposy = m_selstarty;
	
	ResetSelection();
	
	
	return tstr;
}

UTFString UTFTextBuffer::Copy()
{
	UTFString tstr;
	
	BAutolock(this);

	if (m_selstartx != m_selendx || m_selstarty != m_selendy)
	{
		UTFString* ptstr = NULL;
		UTFString lhs;
		UTFString rhs;
		

		if (m_selstarty == m_selendy)
		{
			if (m_selstartx < m_selendx)
			{
				ptstr = (UTFString*)m_buffer.ItemAt(m_selstarty);
				ptstr->Split(m_selendx, lhs, rhs);
				lhs.Split(m_selstartx, tstr, rhs);
				tstr = rhs;
			}	
		}
		else
		{
			ptstr = (UTFString*)m_buffer.ItemAt(m_selstarty);
			ptstr->Split(m_selstartx, lhs, rhs);
			tstr = rhs + "\n";
			ptstr = NULL;
		
			for(long i = m_selstarty + 1; i < m_selendy; i++)
			{
				ptstr = (UTFString*)m_buffer.ItemAt(i);
				tstr += *ptstr;
				tstr += "\n";
				ptstr = NULL;
			}

			ptstr = (UTFString*)m_buffer.ItemAt(m_selendy);
			ptstr->Split(m_selendx, lhs, rhs);
			tstr += lhs;
			tstr += "\n";
			ptstr = NULL;
			
		}
	}
	
	//ResetSelection();
	

	return tstr;
}

void UTFTextBuffer::Paste(const UTFString& PasteString)
{
	BAutolock(this);

	
	Cut();
	
	UTFString* ptstr = NULL;
	
	if (0 >= m_buffer.CountItems())
	{
		ptstr = new UTFString();
	}
	else
	{
		ptstr = (UTFString*)m_buffer.RemoveItem(m_carposy);
	}

	UTFString lhs;
	UTFString rhs;
	
	ptstr->Split(m_carposx, lhs, rhs);
	
	*ptstr = lhs + PasteString + rhs;
	
	UTFStringTokenizer st;
	
	st.SetIgnoreString("\n");
	st.SetDelimiterString("\n");
	st.SetTokenString(*ptstr);
	
	delete ptstr;
	
	BList tlist;
	
	tlist = st.Tokenize();
	
	uint32 cpy = m_carposy;
	
	for (long i = 0; ptstr = (UTFString*)tlist.ItemAt(i); i++)
	{
		UTFString* ptstr2 = new UTFString(*ptstr);
		
		//printf("String: %s at: %d\n", (const char*)*ptstr2, cpy);
		
		if (cpy >= m_buffer.CountItems())
		{
			m_buffer.AddItem(ptstr2);
		}
		else
		{
			m_buffer.AddItem(ptstr2, cpy);
		}
		cpy++;
	}
	
	ResetSelection();
	
}
	
void UTFTextBuffer::AddRow(const UTFString& RowString)
{
	UTFString *ptstr = new UTFString(RowString);
	
	BAutolock(this);

	m_buffer.AddItem((void*)ptstr);
	
}

void UTFTextBuffer::AddRow(uint32 Row, const UTFString& RowString)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	UTFString *ptstr = new UTFString(RowString);
	
	BAutolock(this);

	m_buffer.AddItem((void*)ptstr, Row);
	
}

UTFString UTFTextBuffer::RemoveRow()
{
	UTFString* ptstr = NULL;
	
	BAutolock(this);

	if (0 >= m_buffer.CountItems())
	{
		UTFString tstr;
		return tstr;
	}
		
	ptstr = (UTFString*)m_buffer.RemoveItem((int32)0);
	

	UTFString tstr(*ptstr);
	
	delete ptstr;
	return tstr;
}

UTFString UTFTextBuffer::RemoveRow(uint32 Row)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	UTFString* ptstr = NULL;
	
	BAutolock(this);

	ptstr = (UTFString*)m_buffer.RemoveItem((int32)Row);
	

	UTFString tstr(*ptstr);
	
	delete ptstr;
	return tstr;
}

uint32 UTFTextBuffer::NumRows()
{
	return m_buffer.CountItems();
}
	
UTFString UTFTextBuffer::GetRow(uint32 Row)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	UTFString* ptstr = NULL;
	
	BAutolock(this);

	ptstr = (UTFString*)m_buffer.ItemAt(Row);
	

	UTFString tstr(*ptstr);
	
	return tstr;
}

void UTFTextBuffer::SetRow(uint32 Row, const UTFString& RowString)
{
	assert(0 <= Row);
	assert(Row < m_buffer.CountItems());
	
	UTFString* ptstr = NULL;
	
	BAutolock(this);

	ptstr = (UTFString*)m_buffer.ItemAt(Row);
	*ptstr = RowString;
	

}

void UTFTextBuffer::CopyTextBuffer(const UTFTextBuffer& tb)
{
	UTFString* ptstrin 		= NULL;
	UTFString* ptstrout 	= NULL;
	
	ClearBuffer();
	
	BAutolock(this);

	for (long i = 0; ptstrin = (UTFString*)tb.m_buffer.ItemAt(i); i++)
	{
		ptstrout = new UTFString(*ptstrin);
		m_buffer.AddItem((void*)ptstrout);
		ptstrout = NULL;
	}
	
}

void UTFTextBuffer::ProcessDataBuffer(BDataIO* buf)
{
	ClearBuffer();

	UTFString tstr;
	char cbuf[MAX_UTF_CHARLEN];
	
	BAutolock(this);

	memset(cbuf, '\0', MAX_UTF_CHARLEN);
	
	while(0 < buf->Read(cbuf, 1))
	{
		uint8 cl = UTF8CHARBYTELEN(cbuf[0]);
		
		if (cl > 1)
			buf->Read(&cbuf[1], cl);
		
		if (cbuf[0] == '\n' || !cbuf[0])
		{
			UTFString* ptstr = new UTFString(tstr);
			
			if (cbuf[0] == '\n')
				*ptstr += cbuf;
				
			m_buffer.AddItem((void*)ptstr);
			tstr = "";
		}
		else
		{
			tstr += cbuf;
		}
		
		memset(cbuf, '\0', MAX_UTF_CHARLEN);
	}

	if (0 < tstr.Length())
	{
		UTFString* ptstr = new UTFString(tstr);
		*ptstr += "\n";
		m_buffer.AddItem((void*)ptstr);
		tstr = "";
	}

	
}

void UTFTextBuffer::ClearBuffer()
{
	UTFString* ptstr 		= NULL;
	
	BAutolock(this);

	
	for (long i = 0; ptstr = (UTFString*)m_buffer.ItemAt(i); i++)
	{
		delete ptstr;
	}

	m_buffer.MakeEmpty();
	
}

void UTFTextBuffer::Init()
{
	//Caret
	m_carposx = 0;
	m_carposy = 0;
	
	//Selection Variables	
	m_selstartx = 0;
	m_selstarty = 0;
	m_selendx = 0;
	m_selendy = 0;

}
